# -*- coding: utf-8 -*-
# SDK info
__version__ = "0.1.11"
__package_name__ = "fastflyer"
__title__ = "FastFlyer — 轻量级API开发框架"

__author__ = "Jagerzhang"
__author_email__ = "im@zhang.ge"

__license__ = "Apache 2.0"

__python_requires__ = ">=3, <4"
__classifiers__ = ["Programming Language :: Python :: 3"]
__url__ = "https://github.com/jagerzhang/fastflyer.git"

__copyright__ = "Copyright © 2024 Tencent. All Rights Reserved. 网络平台部 平台研发中心 版权所有"
__description__ = "FastFlyer 是基于 FastAPI 设计的轻量级 API 开发框架。在 FastAPI 优异特性的基础上集成了一系列开箱即用的组件，\
包括Redis、MySQL、Kafka等，让研发人员只需要专注于业务逻辑的实现，真正开箱即用！"
